==> UnderB-1.01.tar.gz: UnderB-1.01/name.c <==

/*
	Too slow.. rewrote everything!

	Daeshik Kim

	hellcat@korea.slip.umd.edu
	dkim@hq.si.net
*/

/*
Eagles Bulletin Board System
Copyright (C) 1994, Ray Rocker, rrrocker@rock.b11.ingr.com
                                rock@seabass.st.usm.edu
                                72673.2105@compuserve.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "common.h"
#include <malloc.h>

static int nodecmp(i,j)
NAMENODE *i;
NAMENODE *j;
{
  register unsigned char *s1, *s2;
  register int xor;

  s1=i->word;
  s2=j->word;


  for (;*s1||*s2; s1++, s2++) {
    if (!*s1 || !*s2)
	return (*s2 - *s1);
    if (*s1 == *s2)
	continue;
    if (myisalpha(*s1) && myisalpha(*s2)) {
	xor = *s1 ^ *s2;
	if (xor != 32 && xor != 0)
	  return (*s1 - *s2);
    } else {
	return (*s1 - *s2);
    }
  }
  return (0);
}

void sort_list(list)
NAMELIST *list;
{
  /* empty */
  if (list == NULL || *list == NULL || (*list)->size == 0 || (*list)->last == 0)
    return;

  qsort((*list)->data, (*list)->size, sizeof(NAMENODE), nodecmp);
}

void
free_namelist(list)
NAMELIST *list;
{
  register NAMENODE *trav;

  if (list == NULL || *list == NULL)
    return;
  free((*list)->data);
  *list = NULL;
  return;
}

void
create_namelist(list, size)
NAMELIST *list;
int size;
{
  NAMELIST ptr;

  if (*list)
  {
    free_namelist(list);
  }
  ptr = (NAMELIST)malloc(sizeof(*ptr));
  ptr->data = (NAMENODE*)calloc(size, sizeof(NAMENODE));
  ptr->size = size;
  ptr->last = 0;
  *list = ptr;
}

add_namelist(list, name, name_before)
NAMELIST *list;
char *name;
char *name_before;
{
  register int size, last;
  register NAMENODE *data;

  if (*list == NULL)
    return S_SYSERR;

  data = (*list)->data;
  size = (*list)->size;
  last = (*list)->last;

  if (last >= size)
    return S_SYSERR;

  if (name && *name)
  {
    strcpy(data[last].word, name);
  }
  else
  {
    *data[last].word = '\0';
  }
    
  ((*list)->last)++;
  return 0;
}

remove_namelist(list, name)
NAMELIST *list;
char *name;
{
  NAMENODE *prev = NULL;
  register NAMENODE *curr;
  register int i, size, last;

  if (list == NULL || *list == NULL || (*list)->size <= 0 || (*list)->last <=0)
    return S_NOTFOUND;

  curr = (*list)->data;
  size = (*list)->size;
  last = (*list)->last;

  for (i=0; i<last && strcasecmp(curr[i].word,name); i++);

  if (i >= last)
    return S_NOTFOUND;

  *curr[i].word = '\0';

  ((*list)->last)--;
  sort_list(list);
  return S_OK;
}

is_in_namelist(list, name)
NAMELIST list;
char *name;
{
  register NAMENODE *ptr;
  register int i, size;

  if (list == NULL || name == NULL || *name == '\0')
    return 0;
  for (i=0, size=list->last, ptr = list->data; i<size; i++) {
    if (!strcasecmp(ptr[i].word, name))
      return 1;
  }
  return 0;
}

apply_namelist(list, fptr, arg)
NAMELIST list;
int (*fptr)();
void *arg;
{
  register int i, size;
  register NAMENODE *ptr;

  for(i=0, size=list->last, ptr=list->data; i<size && ptr; i++)
    (*fptr)(i, ptr[i].word, arg);
  return i;
}

/*ARGSUSED*/
static int _write_list_element(indx, name, fp)
int indx;
char *name;
FILE *fp;
{
  fprintf(fp, "%s\n", name);
  return S_OK;
}

write_namelist(fname, list)
char *fname;
NAMELIST list;
{
  FILE *fp;

  if (list == NULL) unlink(fname);
  else {
    if ((fp = fopen(fname, "w")) == NULL) {
      return S_SYSERR;
    }
    apply_namelist(list, _write_list_element, fp);
    fclose(fp);
  }
  return S_OK;
}

read_namelist(fname, list)
char *fname;
NAMELIST *list;
{
  FILE *fp;
  char rec[NAMELEN+1];	/* this should be enough for reading user names */
  register char *ptr;

  create_namelist(list, MAX_TMP);

  if ((fp = fopen (fname, "r")) == NULL)
    return S_SYSERR;

  while (fgets(rec, NAMELEN, fp)) {
    if (*rec == '#' || myisspace(*rec)) continue;
    for (ptr = rec; (*ptr) && (!myisspace(*ptr)); ptr++);
    *ptr = '\0';
    if (!is_in_namelist(*list, rec)) {
      add_namelist(list, rec, NULL);
    }
  }
  fclose(fp);

  sort_list(list);
  return S_OK;
}

==> UnderB-1.091.tar.gz: UnderB-1.091/name.c <==

/*
	Too slow.. rewrote everything!

	Daeshik Kim

	hellcat@korea.slip.umd.edu
	dkim@hq.si.net
*/

/*
Eagles Bulletin Board System
Copyright (C) 1994, Ray Rocker, rrrocker@rock.b11.ingr.com
                                rock@seabass.st.usm.edu
                                72673.2105@compuserve.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "common.h"
#include <malloc.h>

static int nodecmp(i,j)
NAMENODE *i;
NAMENODE *j;
{
  register unsigned char *s1, *s2;
  register int xor;

  s1=i->word;
  s2=j->word;


  for (;*s1||*s2; s1++, s2++) {
    if (!*s1 || !*s2)
	return (*s2 - *s1);
    if (*s1 == *s2)
	continue;
    if (myisalpha(*s1) && myisalpha(*s2)) {
	xor = *s1 ^ *s2;
	if (xor != 32 && xor != 0)
	  return (*s1 - *s2);
    } else {
	return (*s1 - *s2);
    }
  }
  return (0);
}

void sort_list(list)
NAMELIST *list;
{
  /* empty */
  if (list == NULL || *list == NULL || (*list)->size == 0 || (*list)->last == 0)
    return;

  qsort((*list)->data, (*list)->size, sizeof(NAMENODE), nodecmp);
}

void
free_namelist(list)
NAMELIST *list;
{
  register NAMENODE *trav;

  if (list == NULL || *list == NULL)
    return;
  free((*list)->data);
  *list = NULL;
  return;
}

void
create_namelist(list, size)
NAMELIST *list;
int size;
{
  NAMELIST ptr;

  if (list && *list)
  {
    free_namelist(list);
  }
  ptr = (NAMELIST)malloc(sizeof(*ptr));
  ptr->data = (NAMENODE*)calloc(size, sizeof(NAMENODE));
  ptr->size = size;
  ptr->last = 0;
  *list = ptr;
}

add_namelist(list, name, name_before)
NAMELIST *list;
char *name;
char *name_before;
{
  register int size, last;
  register NAMENODE *data;

  if (*list == NULL)
    return S_SYSERR;

  data = (*list)->data;
  size = (*list)->size;
  last = (*list)->last;

  if (last >= size)
    return S_SYSERR;

  if (name && *name)
  {
    strcpy(data[last].word, name);
  }
  else
  {
    *data[last].word = '\0';
  }
    
  ((*list)->last)++;
  return 0;
}

remove_namelist(list, name)
NAMELIST *list;
char *name;
{
  NAMENODE *prev = NULL;
  register NAMENODE *curr;
  register int i, size, last;

  if (list == NULL || *list == NULL || (*list)->size <= 0 || (*list)->last <=0)
    return S_NOTFOUND;

  curr = (*list)->data;
  size = (*list)->size;
  last = (*list)->last;

  for (i=0; i<last && strcasecmp(curr[i].word,name); i++);

  if (i >= last)
    return S_NOTFOUND;

  *curr[i].word = '\0';

  ((*list)->last)--;
  sort_list(list);
  return S_OK;
}

is_in_namelist(list, name)
NAMELIST list;
char *name;
{
  register NAMENODE *ptr;
  register int i, size;

  if (list == NULL || name == NULL || *name == '\0')
    return 0;
  for (i=0, size=list->last, ptr = list->data; i<size; i++) {
    if (!strcasecmp(ptr[i].word, name))
      return 1;
  }
  return 0;
}

apply_namelist(list, fptr, arg)
NAMELIST list;
int (*fptr)();
void *arg;
{
  register int i, size;
  register NAMENODE *ptr;

  for(i=0, size=list->last, ptr=list->data; i<size && ptr; i++)
    (*fptr)(i, ptr[i].word, arg);
  return i;
}

/*ARGSUSED*/
static int _write_list_element(indx, name, fp)
int indx;
char *name;
FILE *fp;
{
  fprintf(fp, "%s\n", name);
  return S_OK;
}

write_namelist(fname, list)
char *fname;
NAMELIST list;
{
  FILE *fp;

  if (list == NULL) unlink(fname);
  else {
    if ((fp = fopen(fname, "w")) == NULL) {
      return S_SYSERR;
    }
    apply_namelist(list, _write_list_element, fp);
    fclose(fp);
  }
  return S_OK;
}

read_namelist(fname, list)
char *fname;
NAMELIST *list;
{
  FILE *fp;
  char rec[ADDRLEN+1];	/* this should be enough for reading user names */
  register char *ptr;

  create_namelist(list, MAX_TMP);

  if ((fp = fopen (fname, "r")) == NULL)
    return S_SYSERR;

  while (fgets(rec, ADDRLEN, fp)) {
    if (*rec == '#' || myisspace(*rec)) continue;
    for (ptr = rec; (*ptr) && (!myisspace(*ptr)); ptr++);
    *ptr = '\0';
    if (!is_in_namelist(*list, rec)) {
      add_namelist(list, rec, NULL);
    }
  }
  fclose(fp);

  sort_list(list);
  return S_OK;
}
